﻿using System;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

namespace MiddlemarchReader
{
    public partial class MainPage : PhoneApplicationPage
    {
        AppSettings appSettings = (Application.Current as App).AppSettings;
        BookInfo book;

        public MainPage()
        {
            InitializeComponent();

            // Set the fields of the appbar buttons and menu
            chaptersIconButton = this.ApplicationBar.Buttons[0] as ApplicationBarIconButton;
            slidingMenuItem = this.ApplicationBar.MenuItems[0] as ApplicationBarMenuItem;
            curlingMenuItem = this.ApplicationBar.MenuItems[1] as ApplicationBarMenuItem;
            flippingMenuItem = this.ApplicationBar.MenuItems[2] as ApplicationBarMenuItem;

            switch (appSettings.PageTransition)
            {
                case "slide": OnAppBarMenuItemClick(slidingMenuItem, EventArgs.Empty); break;
                case "curl": OnAppBarMenuItemClick(curlingMenuItem, EventArgs.Empty); break;
                case "flip": OnAppBarMenuItemClick(flippingMenuItem, EventArgs.Empty); break;
            }

            book = appSettings.Book;

            Loaded += (object sender, RoutedEventArgs args) => 
            {
                bookViewer.JumpToPage(book.CurrentChapterIndex, book.CurrentPageIndex); 
            };
        }

        void OnChaptersListBoxSelectionChanged(object sender, SelectionChangedEventArgs args)
        {
            ChapterInfo chapter = chaptersListBox.SelectedItem as ChapterInfo;

            if (chapter != null)
            {
                book.CurrentChapterIndex = book.Chapters.IndexOf(chapter);
                book.CurrentPageIndex = 0;
                bookViewer.JumpToPage(book.CurrentChapterIndex, book.CurrentPageIndex);
            }
            SwitchToBookView();
        }

        void OnAppBarChaptersButtonClick(object sender, EventArgs args)
        {
            chaptersListBox.SelectedItem = null;

            if (chaptersListBox.Visibility == Visibility.Visible)
                SwitchToBookView();
            else
                SwitchToChaptersView();
        }

        void SwitchToBookView()
        {
            chaptersIconButton.IconUri = new Uri("Images/chapters.icon.png", UriKind.Relative);
            chaptersIconButton.Text = "chapters";
            chaptersListBox.Visibility = Visibility.Collapsed;
            bookViewer.Visibility = Visibility.Visible;
        }

        void SwitchToChaptersView()
        {
            chaptersIconButton.IconUri = new Uri("Images/text.icon.png", UriKind.Relative);
            chaptersIconButton.Text = "text";
            chaptersListBox.Visibility = Visibility.Visible;
            bookViewer.Visibility = Visibility.Collapsed;
        }

        void OnAppBarMenuItemClick(object sender, EventArgs args)
        {
            foreach (ApplicationBarMenuItem menuItem in this.ApplicationBar.MenuItems)
            {
                char ch = sender == menuItem ? '\x25C9' : '\x25EF';
                menuItem.Text = ch + menuItem.Text.Substring(1);
            }

            if (sender == slidingMenuItem)
            {
                appSettings.PageTransition = "slide";
                bookViewer.PageTransition = new SlideTransition();
            }

            else if (sender == curlingMenuItem)
            {
                appSettings.PageTransition = "curl";
                bookViewer.PageTransition = new CurlTransition();
            }

            else if (sender == flippingMenuItem)
            {
                appSettings.PageTransition = "flip";
                bookViewer.PageTransition = new FlipTransition();
            }
        }

        void OnBookViewerPageChanged(object sender, EventArgs args)
        {
            int chapterIndex = bookViewer.ChapterIndex;
            int pageIndex = bookViewer.PageIndex;

            if (chapterIndex < book.Chapters.Count && pageIndex < book.Chapters[chapterIndex].Pages.Count)
            {
                // Save for isolated storage settings
                book.CurrentChapterIndex = bookViewer.ChapterIndex;
                book.CurrentPageIndex = bookViewer.PageIndex;

                // Update the percentage indicator
                int accumulatedCharacterCount = book.Chapters[chapterIndex].Pages[pageIndex].AccumulatedCharacterCount;
                double percentage = 100.0 * accumulatedCharacterCount / book.CharacterCount;
                percentageRun.Text = String.Format("{0:F1}%", percentage);
            }
        }
    }
}
